from pathlib import Path
from typing import List

from .lib import Base


class SuperGroups(Base):
    def create(self, name: str, members: List, topics: List):
        return self._make_request('supergroups.create', payload={
            'supergroups': [
                {
                    'fname': name,
                    'members': members,
                    'topics': topics,
                },
            ],
        })

    def add_topics(self, room_id: str, new_topic_name: str):
        return self._make_request('supergroups.addTopics', payload={
            'roomId': room_id,
            'topics': [
                {
                    'fname': new_topic_name,
                },
            ],
        })

    def remove_topics(self, room_id: str, topics: List[str]):
        return self._make_request('supergroups.removeTopics', payload={
            'roomId': room_id,
            'topicIds': topics,
        })

    def get_topics(self, room_id: str):
        return self._make_request(
            'supergroups.topics',
            params={
                'roomId': room_id,
                'sequence': 0,
                'limit': 50,
            },
            method='get',
        )

    def edit_topic(self, topic_id: str, new_topic_name: str):
        return self._make_request('supergroups.editTopics', payload={
            'topics': [
                {
                    '_id': topic_id,
                    'fname': new_topic_name,
                },
            ],
        })

    def invite(self, room_id: str, user_ids: List[str]):
        return self._make_request('supergroups.invite', payload={
            'roomId': room_id,
            'userIds': user_ids,
        })

    def set_avatar(self, sgp_id: str, file_path: Path):
        files = {'file': open(file_path, 'rb')}
        return self._make_request(
            f'supergroups.setAvatar/{sgp_id}', files=files,
        )

    def rename(self, sgp_id: str, new_name: str):
        return self._make_request('supergroups.rename', payload={
            'roomId': sgp_id,
            'name': new_name,
        })

    def add_owner(self, sgp_id: str, user_id: str):
        return self._make_request('supergroups.addOwner', payload={
            'roomId': sgp_id,
            'userId': user_id,
        })

    def remove_owner(self, sgp_id: str, user_id: str):
        return self._make_request('supergroups.removeOwner', payload={
            'roomId': sgp_id,
            'userId': user_id,
        })

    def convert_to_group(self, sgp_id: str):
        return self._make_request(
            'supergroups.convertToGroup', payload={'roomId': sgp_id},
        )



from .lib import Base


class Users(Base):
    def info(self, username: str):
        return self._make_request(
            'users.info', params={'username': username}, method='get',
        )

    def search_for_room_mention(
            self,
            room_id: str,
            query: str = '',
            members_only: int = 1,
    ):
        return self._make_request(
            'users.searchForRoomMention',
            params={
                'roomId': room_id,
                'query': query,
                'count': self.settings.count,
                'offset': self.settings.offset,
                'searchInMembersOnly': members_only,
            },
            method='get',
        )

    def me(self):
        return self._make_request(
            'me', method='get',
        )



from .lib import Base


class DmIm(Base):
    def im_create_username(self, username: str):
        return self._make_request('im.create', payload={'username': username})

    def im_members(self, room_id: str):
        return self._make_request(
            'im.members',
            params={
                'roomId': room_id,
                'count': self.settings.count,
                'offset': self.settings.offset,
            },
            method='get',
        )

    def dm_create_user_id(self, user_id: str):
        return self._make_request('dm.create', payload={'userId': user_id})

    def dm_delete(self, room_id: str, type: str):
        return self._make_request('dm.delete', payload={
            'roomId': room_id,
            'type': type,  # NOTE: "soft" | "hard"
        })



from ..constance import Params
from .avatar import AvatarAPI
from .chat import ChatAPI
from .config import Settings
from .different import DifferentAPI
from .dm_im import DmImAPI
from .folders import FoldersAPI
from .groups import GroupsAPI
from .polls import PollsAPI
from .rooms import RoomsAPI
from .supergroups import SuperGroupsAPI
from .testing import TestingAPI  # noqa
from .threads import ThreadAPI
from .users import UsersAPI


class ClientAPI():
    def __init__(self, session):
        self.session = session
        self.base_url = Params.REMOTE_URL
        self.settings = Settings(config_file='settings.ini')
        self.avatar = AvatarAPI(self)
        self.dm_im = DmImAPI(self)
        self.groups = GroupsAPI(self)
        self.polls = PollsAPI(self)
        self.rooms = RoomsAPI(self)
        self.supergroups = SuperGroupsAPI(self)
        self.folders = FoldersAPI(self)
        self.users = UsersAPI(self)
        self.different = DifferentAPI(self)
        self.chat = ChatAPI(self)
        self.threds = ThreadAPI(self)



from typing import List

from .lib import Base


class Folders(Base):
    def list(self):
        return self._make_request(
            'folders.list', method='get',
        )

    def create(self, name: str, rooms: List[str]):
        return self._make_request('folders.create', payload={
            'folderName': name,
            'rooms': rooms,
        })

    def add_room(self, folder_id: str, rooms: List[str]):
        return self._make_request('folders.addRoom', payload={
            'folderId': folder_id,
            'roomIds': rooms,
        })

    def remove_room(self, folder_id: str, rooms: List[str]):
        return self._make_request('folders.removeRoom', payload={
            'folderId': folder_id,
            'roomIds': rooms,
        })

    def pinned_room(self, folder_id: str, rooms: List[str]):
        return self._make_request('folders.savePinnedRooms', payload={
            'folderId': folder_id,
            'pinnedRoomIds': rooms,
        })

    def update(self, folder_id: str, name: str):
        return self._make_request('folders.update', payload={
            'folderId': folder_id,
            'folderName': name,
        })

    def remove(self, folder_id: str):
        return self._make_request(
            'folders.remove', payload={'folderId': folder_id},
        )



from typing import Any, Union

import requests

from .lib import Base


class Different(Base):
    def version(self):
        return requests.get(f'{self.short_url}/info', verify=False)

    def preferences_list(self):
        return self._make_request('preferences.list', method='get')

    def preferences_set(self, preference: str, value: Union[int, str]):
        return self._make_request(
            'preferences.set', payload={preference: value},
        )

    def timesync(self):
        return requests.get(f'{self.short_url}/timesync', verify=False)

    def settings_public(self):
        return self._make_request(
            'settings.public',
            params={
                'offset': self.settings.offset,
                'count': self.settings.count,
            },
            method='get',
        )

    def setting_public_by_id(self, setting_id: str):
        return self._make_request(
            f'settings.public/{setting_id}', method='get',
        )

    def settings_set(self, setting_id: str, value: Any):
        return self._make_request(
            'settings.set', payload={'id': setting_id, 'value': value},
        )

    def subscriptions_read(self, room_id: str):
        return self._make_request(
            'subscriptions.read', payload={'rid': room_id},
        )



from requests import Response

from .lib import Base


class Spotlight(Base):
    """Сервис Spotlight."""

    def search_by_spotlight(self, page_count: int, search_query: str,
                            search_schema: list) -> Response:
        """Поиск в Spotlight по определенной схеме."""
        return self._make_request(
            'spotlight',
            payload={
                'count': page_count,
                'query': search_query,
                'searchSchema': search_schema,
            },
        )



from typing import List

from .lib import Base


class Groups(Base):
    def create(self, name: str, members: List[str]):
        return self._make_request('groups.create', payload={
            'name': name,
            'members': members,
        })

    def info(self, room_id: str):
        return self._make_request(
            'groups.info',
            params={
                'roomId': room_id,
                'offset': self.settings.offset,
                'count': self.settings.count,
            },
            method='get',
        )

    def members(self, room_id: str):
        return self._make_request(
            'groups.members',
            params={
                'roomId': room_id,
                'offset': self.settings.offset,
                'count': self.settings.count,
            },
            method='get',
        )

    def invite(self, room_id: str, user_id: str):
        return self._make_request('groups.invite', payload={
            'roomId': room_id,
            'userId': user_id,
        })

    def delete(self, room_id: str):
        return self._make_request('groups.delete', payload={'roomId': room_id})

    def rename(self, room_id: str, name: str):
        return self._make_request('groups.rename', payload={
            'roomId': room_id,
            'name': name,
        })

    def set_avatar(self, group_id: str, filename: str):
        files = {'file': open(filename, 'rb')}
        return self._make_request(f'groups.setAvatar/{group_id}', files=files)

    def convert_to_super(self, room_id: str, topic_name: str, emoji: str):
        payload = {
            'roomId': room_id,
            'defaultTopic': {
                'fname': topic_name,
                'emoji': emoji,
            },

        }
        return self._make_request(
            'groups.convertToSupergroup', payload=payload,
        )

    def add_owner(self, room_id: str, user_id: str):
        return self._make_request(
            'groups.addOwner', payload={
                'roomId': room_id,
                'userId': user_id,
            },
        )

    def kick(self, room_id: str, user_id: str):
        return self._make_request(
            'groups.kick', payload={
                'roomId': room_id,
                'userId': user_id,
            },
        )



from .lib import Base


class Avatar(Base):
    def get_avatar(self, user_id: str):
        return self._make_request(
            f'/avatar/{user_id}',
            params={
                'etag': 'DEFAULT',
            },
            method='get',
        )

    def get_avatar_user(self, user_id: str):
        return self._make_request(
            f'/avatar/user/{user_id}', method='get',
        )

    def get_avatar_room(self, room_id: str):
        return self._make_request(
            f'/avatar/room/{room_id}', method='get',
        )



from datetime import datetime
from typing import List, Optional

from pydantic import BaseModel, Field


class IdMixin(BaseModel):
    id: str = Field(alias='_id')


class UpdatedAtMixin(BaseModel):
    updatedAt: Optional[datetime] = Field(None, alias='_updatedAt')


class TypeMixin(BaseModel):
    type: str


class UserModel(IdMixin, TypeMixin, UpdatedAtMixin):
    active: bool
    aliasId: str
    createdAt: str
    domain: str
    name: str
    notificationsDisabled: bool = False
    preferences: List = []
    roles: List[str]
    username: str


class AdminBotModel(IdMixin, TypeMixin, UpdatedAtMixin):
    active: bool
    createdAt: str
    name: str
    notificationsDisabled: bool = False
    preferences: List = []
    roles: List[str]
    username: str


class AuthData(BaseModel):
    userId: str
    authToken: str
    authTokenExpires: int
    me: UserModel | AdminBotModel


class AuthResponse(BaseModel):
    status: str
    data: AuthData



from http import HTTPStatus

import requests

from .scheme import AuthResponse


def auth(username, password, base_url):
    session = requests.Session()
    login_data = {
        'user': username,
        'password': password,
    }
    response = session.post(
        f'{base_url}/login',
        json=login_data,
        verify=False,
    )
    assert response.status_code == HTTPStatus.OK
    auth_response = AuthResponse(**response.json())
    token = auth_response.data.authToken
    uid = auth_response.data.me.id
    return token, uid


class Auth():
    def session(self, username, password):
        auth_token, uid = auth(username, password, self.base_url)
        session = requests.Session()
        headers = {
            'X-Auth-Token': auth_token,
            'X-User-Id': uid,
        }
        session.headers.update(headers)
        session.username = username
        session.uid = uid
        return session



import uuid
from typing import List

from .lib import Base


class Chat(Base):
    def send_message(self, text: str, room_id: str):
        _uuid = str(uuid.uuid4())
        return self._make_request(
            'chat.sendMessage', payload={
                'message': {
                    'rid': room_id,
                    'msg': text,
                    '_id': _uuid,
                },
            },
        ), _uuid

    def message_reply(self, text: str, room_id: str, message_id: str):
        return self._make_request(
            'chat.sendMessage', payload={
                'message': {
                    'rid': room_id,
                    'msg': text,
                    'toReplyId': message_id,
                },
            },
        )

    def forward_message(
            self,
            room_id: str,
            to_room_id: str,
            message_ids: List[str],
            updated_ids: List[str],
    ):
        return self._make_request(
            'chat.forwardMessages', payload={
                'roomId': room_id,
                'toForwardRoomId': to_room_id,
                'toForwardIds': message_ids,
                'forwardedIds': updated_ids,
            },
        )

    def load_message_history(self, room_id: str):
        return self._make_request(
            'chat.loadMessageHistory', payload={'roomId': room_id},
        )

    def pin(self, message_id: str):
        return self._make_request(
            'chat.pinMessage', payload={'messageId': message_id},
        )

    def unpin(self, message_id: str):
        return self._make_request(
            'chat.unPinMessage', payload={'messageId': message_id},
        )

    def get_by_id(self, message_id: str):
        return self._make_request(
            'chat.getMessage', params={'msgId': message_id}, method='get',
        )

    def get_pinned(self, room_id: str):
        return self._make_request(
            'chat.getPinnedMessages',
            params={
                'roomId': room_id,
                'offset': self.settings.offset,
                'count': self.settings.count,
            },
            method='get',
        )

    def react(
            self,
            emoji: str,
            message_id: str,
            room_id: str,
            should_react: bool,
    ):
        return self._make_request(
            'chat.react', payload={
                'emoji': emoji,
                'messageId': message_id,
                'roomId': room_id,
                'shouldReact': should_react,
            },
        )

    def get_reactions(
            self,
            message_id: str,
            room_id: str,
    ):
        return self._make_request(
            'chat.messageReactions',
            params={
                'msgId': message_id,
                'roomId': room_id,
                'offset': self.settings.offset,
                'count': self.settings.count,
            },
            method='get',
        )

    def search(self, text: str, room_id: str):
        return self._make_request(
            'chat.search',
            params={
                'searchText': text,
                'roomId': room_id,
            },
            method='get',
        )

    def read_users(
            self,
            message_id: str,
            room_id: str,
    ):
        return self._make_request(
            'chat.messageReadUsers',
            params={
                'msgId': message_id,
                'roomId': room_id,
                'count': self.settings.count,
                'offset': self.settings.offset,
            },
            method='get',
        )

    def draft_post(self, room_id: str, text: str):
        return self._make_request('chat.draft', payload={
            'roomId': room_id,
            'msg': text,
            'mode': {
                'type': 'none',
                'data': None,
            },
        })

    def draft_get(self, room_id: str):
        return self._make_request(
            'chat.draft', params={'roomId': room_id}, method='get',
        )



import mimetypes
import uuid
from pathlib import Path
from typing import List, Literal, Optional, Union

from .lib import Base


class Rooms(Base):
    def members(self, room_id: str):
        return self._make_request(
            'rooms.members',
            params={
                'roomId': room_id,
                'count': self.settings.count,
                'offset': self.settings.offset,

            },
            method='get',
        )

    def room_sync_preview(
            self,
            hidden_rooms: str = 'include',
            room_types: List[str] = ['d', 'p', 'tc'],
            alerted_only: bool = False,
            updated_since: int = 0,
            sort_lm: int = 1,
            sortstarred: int = 1,
            folder_id: str = 'all',
            exclude: List[str] = [],
    ):
        return self._make_request('rooms.sync', payload={
            'preview': {
                'hidenRooms': hidden_rooms,
                'roomTypes': room_types,
                'alertedOnly': alerted_only,
                'updatedSince': updated_since,
                'sortlm': sort_lm,
                'sortstarred': sortstarred,
                'offset': self.settings.offset,
                'count': self.settings.count,
                'except': exclude,
                'folderId': folder_id,
            },
        })

    def room_sync_list(
            self,
            alerted_only: bool = False,
            folder_id: str = '',
            exclude: List[Optional[str]] = [],
            room_type: List[
                Literal['d', 'p', 'tc', 's']
            ] = ['d', 'p', 'tc', 's'],
    ):
        return self._make_request('rooms.sync', payload={
            'list': {
                'alertedOnly': alerted_only,
                'offset': self.settings.offset,
                'count': self.settings.count,
                'except': exclude,
                'folderId': folder_id,
                'roomTypes': room_type,
            },
        })

    def room_sync_required(
            self,
            req: str,
            objects: List[str] = [],
    ):
        if req == 'saved':
            payload = {
                'required': {
                    'saved': True,
                },
            }
        if req == 'support':
            payload = {
                'required': {
                    'support': True,
                },
            }
        if req == 'byIds':
            payload = {
                'required': {
                    'byIds': objects,
                },
            }
        if req == 'byRoomName':
            payload = {
                'required': {
                    'byRoomName': objects,
                },
            }
        return self._make_request('rooms.sync', payload=payload)

    def room_sync_alerts(self):
        return self._make_request(
            'rooms.sync',
            payload={
                'alerts':
                {
                    'updatedSince': 0,
                },
            })

    def room_sync_alerted_rooms(self):
        return self._make_request(
            'rooms.sync',
            payload={
                'alertedRooms':
                {
                    'total': True,
                },
            })

    def room_sync_removed(self):
        return self._make_request('rooms.sync', payload={
            'removed': {
                'deletedSince': 0,
                'offset': self.settings.offset,
                'count': self.settings.count,
            },
        })

    def room_sync_info(self, room_id: str):
        return self._make_request(
            'rooms.info', params={'roomId': room_id}, method='get',
        )

    def leave(self, room_id: str):
        return self._make_request(
            'rooms.leave',
            payload={'roomId': room_id},
        )

    def set_notifications(self, room_id: str, mute: bool = False):
        return self._make_request(
            'rooms.setNotifications',
            payload={'roomId': room_id, 'mute': mute},
        )

    def my_rooms(self):
        return self._make_request(
            'rooms.myRoomsWithOtherUser',
            params={
                'userId': self.client.session.uid,
                'offset': self.settings.offset,
                'count': self.settings.count,
            },
            method='get',
        )

    def search_in_my_rooms(self):
        return self._make_request(
            'rooms.searchInMyRooms',
            params={
                'offset': self.settings.offset,
                'count': self.settings.count,
            },
            method='get',
        )

    def setting(self, room_id: str, arg: str = '', value: str = ''):
        return self._make_request(
            'rooms.saveRoomSettings',
            payload={
                'rid': room_id,
                arg: value,
            },
        )

    def pin(self, folder_id: str, room_id: str, position: int = 0):
        return self._make_request(
            'rooms.pin', payload={
                'folderId': folder_id,
                'roomId': room_id,
                'position': position,
            },
        )

    def unpin(self, folder_id: str, room_id: str):
        return self._make_request(
            'rooms.pin', payload={
                'folderId': folder_id,
                'roomId': room_id,
            },
        )

    def get_mime_type(self, path: Path):
        mime_type, _ = mimetypes.guess_type(path)
        return mime_type or 'application/octet-stream'

    def _upload_file_base(
            self,
            endpoint: str,
            room_id: str,
            path: Path,
            text: str,
            extra_data: Optional[dict] = None,
    ):
        path = Path(path) if not isinstance(path, Path) else path
        mtype = self.get_mime_type(path)
        data = {}
        if text:
            data['msg'] = text
        if extra_data:
            data.update(extra_data)
        with open(path, 'rb') as file_obj:
            files = {'file': (path.name, file_obj, mtype)}
            return self.client.session.post(
                f'{self.client.base_url}/{endpoint}/{room_id}',
                files=files,
                verify=False,
                data=data,
            )

    def upload_file(self, room_id: str, text: str, path: Path):
        return self._upload_file_base('rooms.upload', room_id, path, text)

    def upload_speech(
            self,
            room_id: str,
            text: str,
            path: Path,
            reply_id: Union[str, None] = None,
    ):
        extra_data = {'_id': str(uuid.uuid4())}
        if reply_id:
            extra_data['toReplyId'] = reply_id
        return self._upload_file_base(
            'rooms.uploadSpeech', room_id, path, text, extra_data,
        )



from setuptools import find_packages, setup


def readme():
    with open('README.md', 'r') as f:
        return f.read()


setup(
    name='ecss_chat_client',
    version='1.1.6',
    author='maxstolpovskikh',
    author_email='maximstolpovskikh@gmail.com',
    description='This is the simplest module HTTP API EcssChatServer',
    long_description=readme(),
    long_description_content_type='text/markdown',
    packages=find_packages(),
    install_requires=['requests>=2.32.3'],
    classifiers=[
        'Programming Language :: Python :: 3.12',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    keywords='ecss_chat_client',
    python_requires='>=3.12.3',
)



from .lib import Base


class Testing(Base):
    def new_user(
            self,
            username,
            name: str,
            password: str,
            domain: str,
    ):
        response = self._make_request(
            'testing.newUser',
            payload={
                'username': username,
                'name': name,
                'password': password,
                'domain': domain,
            },
        )
        return response
